
#ifndef _ALLSTUFF_H
#define _ALLSTUFF_H



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>


#include <allegro.h>
#include <alleggl.h>
#include <GL/glu.h>



#include <windows.h>

#include <vector>
using namespace std;


#undef assert
#define assert(v)	if(!(v)) allegro_message("assert: %s (%d): %s",__FILE__,__LINE__,#v);

#define do_once(v)	\
	static int _once=0;	\
	if(!_once) { _once=1; v; }


#define msg(v)	allegro_message v
#define msg_once(v)  do_once( msg(v) )


struct vec3 {
	float x,y,z;
	
	vec3() {}
	vec3(float _x,float _y,float _z) : x(_x), y(_y), z(_z) {}
	vec3 operator +(const vec3 &v) const { return vec3(x+v.x,y+v.y,z+v.z); }
	vec3 operator -(const vec3 &v) const { return vec3(x-v.x,y-v.y,z-v.z); }
	vec3 operator *(float s) const { return vec3(x*s,y*s,z*s); }
	void operator +=(const vec3 &v) { x+=v.x; y+=v.y; z+=v.z; }
	void operator -=(const vec3 &v) { x-=v.x; y-=v.y; z-=v.z; }
	void operator *=(float s) { x*=s; y*=s; z*=s; }
	
	float dot(const vec3 &v) const { return x*v.x+y*v.y+z*v.z; }
	
	float length() const { return sqrt(x*x+y*y+z*z); }

	void normalize()
	{
		float l = length();
		if(l<=0) return;
		x/=l;
		y/=l;
		z/=l;
	}
};


// engine.cpp
struct CTexture {
	int id,base_w,base_h;
	char name[20];
	
	CTexture() : id(0) { name[0]=0; }
	~CTexture() { glDeleteTextures(1,(GLuint*)&id); }
	
	bool CTexture::Load(char *name);
	void CTexture::Bind(int unit);
	void CTexture::UnBind(int unit);
};


extern vec3 cam_pos;
extern float cam_ax,cam_ay;
extern int render_frame;

void set_3d_mode();
void set_2d_mode();
void init_engine();
void deinit_engine();
CTexture *load_texture(char *name);
void render_map();
void draw_mouse();


// mesh.cpp
#include "mesh.h"


// map.cpp
#define MAP_SIZE		32

#define T_WALL		0
#define T_FLOOR		1
#define T_DETAIL	2


struct tObject;

struct tTile {
	int type;
	int mark;
	tObject *first,*last;
	int render_frame;
};


struct CMap {
	tTile tiles[MAP_SIZE][MAP_SIZE];
	int f_tiles;
	int n_marked;
	int start_x,start_y;
	int end_x,end_y;

	void CMap::ClearMark();
	void CMap::FloodMark(int x,int y,int t);
	void CMap::Room(int x1,int y1,int x2,int y2,int t);
	void CMap::Make(int seed);
	void CMap::CreateTile(int x,int y);
};

extern CMap map;


void make_map(int seed);
void render_tile(int x,int y);


// hud.cpp
void hud_print(char *s,float x,float y,float h);
float hud_text_length(char *s,float h);
void init_hud();
void deinit_hud();


// parts.cpp
struct tPType;

void render_parts();
void think_parts(float dt);
void clear_parts();
void init_parts();
void add_parts(tPType *pt,vec3 pos,vec3 vel,float rp,float rv,int cnt,float scale=1);
tPType *part_type(char *name);


// objects.cpp

#define FL_LIGHT			(1<<0)
#define FL_EMIT				(1<<1)
#define FL_BLEND_ADD		(1<<2)
#define FL_BLEND_ALPHA		(1<<3)
#define FL_FULL_BRIGHT		(1<<4)
#define FL_FLARE			(1<<5)
#define FL_WALK				(1<<6)
#define FL_GRAVITY			(1<<7)
#define FL_STATIC			(1<<8)
#define FL_NO_COLLISION		(1<<9)
#define FL_DELETED			(1<<10)
#define FL_TRIGGER			(1<<11)
#define FL_FAST_EMIT		(1<<12)


#define S_STAND		0
#define S_RUN		1
#define S_ATTACK	2
#define S_DIE		3



typedef void tThinkFunc(tObject *o);


struct tObject {
	vec3	pos,vel;
	int		tile_x,tile_y;
	CMesh	*mesh;
	CTexture *tex;
	float	heading,ideal_heading;
	int		frame;
	vec3	light_color;
	tPType	*ptype;
	float	emit_timer;
	float	ppr,pvr,pscale;
	vec3	pvel;
	float	r,g,b,a;
	int		flags;
	int		state,state2,spell;
	float	health,speed,adelay;
	
	vec3	box_min,box_max;

	int		think_frame;
	tObject *owner;


	tThinkFunc	*think_func;
	float		think_time;
	tThinkFunc	*col_func;

	tObject *next,*prev;
};


extern tObject *player_obj;
extern vec3 mouse_p0;
extern vec3 mouse_p1;
extern tObject *col_object;


void clear_objects();
tObject *create_object();
void destroy_object(tObject *o);
void update_object(tObject *o);
void render_objects(int tx,int ty);
void think_objects(float dt);
void spawn_player();
void cam_follow_player();


// player.cpp
#define MAX_SPELLS		8

extern int spells[MAX_SPELLS];
extern int act_spell;

void launch_spell(vec3 pos,float head,int id,tObject *owner);
void player_think(float dt);
void show_spells();


// think.cpp
extern bool level_complete;
extern int level;
extern int score;


void think_torch(tObject *o);
void spawn_enemy(int tx,int ty);
void spawn_exit(int tx,int ty);
void spawn_cloud(int tx,int ty);



#endif

