
#include "allstuff.h"


vector<CTexture *> textures;
CTexture dummy_tex;

vec3 cam_pos;
float cam_ax,cam_ay;
int render_frame;


CTexture *mouse_tex;



bool CTexture::Load(char *_name)
{
	char path[100];
	strcpy(name,_name);
	sprintf(path,"data/tex/%s.bmp",name);
	BITMAP *bmp = load_bitmap(path,NULL);
	if(!bmp) return false;
	
	sprintf(path,"data/tex/%s_a.bmp",name);
	BITMAP *alpha = load_bitmap(path,NULL);
	int a=(alpha!=NULL);
	if(alpha)
	{
		int x,y;
		for(y=0;y<bmp->h;y++)
			for(x=0;x<bmp->w;x++)
				_putpixel32(bmp,x,y,_getpixel32(bmp,x,y)|(getb32(_getpixel32(alpha,x,y))<<24));
		destroy_bitmap(alpha);
	}
	
	id = allegro_gl_make_texture_ex(AGL_TEXTURE_MIPMAP|(a?AGL_TEXTURE_HAS_ALPHA:0),bmp,-1);
	destroy_bitmap(bmp);
	
	if(!id)
		return false;
	
	return true;
}

void CTexture::Bind(int unit)
{
	glActiveTextureARB(GL_TEXTURE0+unit);
	glBindTexture(GL_TEXTURE_2D,id);
	glEnable(GL_TEXTURE_2D);
}

void CTexture::UnBind(int unit)
{
	glActiveTextureARB(GL_TEXTURE0+unit);
	glDisable(GL_TEXTURE_2D);
}


void set_3d_mode()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(70.0,float(SCREEN_W)/SCREEN_H,0.01,1000);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
//	glTranslatef(0,0,-5);
	glRotatef(-90,1,0,0);
	glRotatef(90,0,0,1);
	glRotatef(-cam_ay,0,1,0);
	glRotatef(-cam_ax,0,0,1);
	glTranslatef(-cam_pos.x,-cam_pos.y,-cam_pos.z);

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glDepthMask(true);

/*	glBegin(GL_TRIANGLES);
#define v(x,y,z)	glColor3f(x,y,z); glVertex3f(x,y,z);
	v(1,0,0) v(0,1,0) v(0,0,1)
	glEnd();*/
	
	glFogi(GL_FOG_MODE,GL_LINEAR);
	glFogf(GL_FOG_DENSITY,0.7);
	glFogf(GL_FOG_START,2.0);
	glFogf(GL_FOG_END,5.0);
	float fc[4]={0,0,0,1};
	glFogfv(GL_FOG_COLOR,fc);
	glEnable(GL_FOG);

	render_frame++;
}

void set_2d_mode()
{
	glDisable(GL_FOG);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	glDisable(GL_DEPTH_TEST);
	glDepthMask(false);
}

void init_engine()
{
	init_hud();
	init_parts();
	load_arbvp();
	init_lighting();
	
	mouse_tex = load_texture("mouse");
}

void deinit_engine()
{
	int i;
	for(i=0;i<textures.size();i++)
		if(textures[i])
			delete textures[i];

	textures.clear();

	unload_meshes();
	deinit_hud();
}

CTexture *load_texture(char *name)
{
	int i;
	for(i=0;i<textures.size();i++)
		if(strcmp(textures[i]->name,name)==0)
			return textures[i];
	CTexture *t = new CTexture();
	if(!t) return &dummy_tex;
	if(!t->Load(name))
	{
//		allegro_message("!");
		delete t;
		return &dummy_tex;
	}
	textures.push_back(t);
	return t;
}

void draw_mouse()
{
	glEnable(GL_BLEND);
//	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glBlendFunc(GL_ONE,GL_ONE);
	mouse_tex->Bind(0);
	
	float xp= float(mouse_x)/SCREEN_W*2-1;
	float yp=-float(mouse_y)/SCREEN_H*2+1;
	float dx=0.05,dy=-0.05;
	glColor4f(1,1,1,1);
	glBegin(GL_QUADS);
#define v(x,y)	glTexCoord2f(x,y); glVertex3f(xp+x*dx,yp+y*dy,0);
	v(0,0); v(0,1); v(1,1); v(1,0);
#undef v
	glEnd();
	mouse_tex->UnBind(0);
	glDisable(GL_BLEND);
}

void render_map()
{
	int x,y,x1,y1,x2,y2;
//	x1=y1=0; x2=y2=31;
	
	x=int(floor(player_obj->pos.x));
	y=int(floor(player_obj->pos.y));
	
	x1=MID(0,x-5,MAP_SIZE-1);
	y1=MID(0,y-5,MAP_SIZE-1);
	x2=MID(0,x+5,MAP_SIZE-1);
	y2=MID(0,y+5,MAP_SIZE-1);
	
	for(y=y1;y<=y2;y++)
		for(x=x1;x<=x2;x++)
		{
//			glPushMatrix();
//			glTranslatef(x,y,0);
			render_tile(x,y);
//			glPopMatrix();
		}
}

