
#include "allstuff.h"



CMap map;


const int DX[4]={-1, 0, 1, 0};
const int DY[4]={ 0,-1, 0, 1};


void CMap::ClearMark()
{
	int x,y;
	for(x=0;x<MAP_SIZE;x++)
		for(y=0;y<MAP_SIZE;y++)
			tiles[x][y].mark=0;
	n_marked=0;
}

void CMap::FloodMark(int x,int y,int t)
{
	if(tiles[x][y].type!=t || tiles[x][y].mark)
		return;
	tiles[x][y].mark=1;
	n_marked++;
	FloodMark(x+1,y  ,t);
	FloodMark(x-1,y  ,t);
	FloodMark(x  ,y+1,t);
	FloodMark(x  ,y-1,t);
}

void CMap::Room(int x1,int y1,int x2,int y2,int t)
{
	int x,y;
	for(y=y1;y<=y2;y++)
		for(x=x1;x<=x2;x++)
		{
			if(tiles[x][y].type!=T_FLOOR)
				f_tiles++;
			tiles[x][y].type=T_FLOOR;
		}
}

void CMap::Make(int seed)
{
	clear_objects();
	clear_parts();

	srand(seed);
	memset(tiles,0,sizeof(tiles));
	f_tiles=0;
	
	int x,y,x1,y1,x2,y2,b,r;
	
	start_x=rand()%(MAP_SIZE-2)+1;
	start_y=rand()%(MAP_SIZE-2)+1;
	while(1)
	{
		end_x=rand()%(MAP_SIZE-2)+1;
		end_y=rand()%(MAP_SIZE-2)+1;
		if(abs(end_x-start_x)>10 || abs(end_y-start_y)>10)
			break;
	}

	while(1)
	{
		b=1;
//		if(b && f_tiles<MAP_SIZE*MAP_SIZE/4) b=0;
		if(b)
		{
			ClearMark();
			FloodMark(start_x,start_y,T_FLOOR);
			if(n_marked<MAP_SIZE*MAP_SIZE/4)
				b=0;
			if(!tiles[end_x][end_y].mark)
				b=0;
		}
		if(b) break;
		
		do
		{
			x1=rand()%(MAP_SIZE-2)+1;
			y1=rand()%(MAP_SIZE-2)+1;
		} while(tiles[x1][y1].type==T_FLOOR);
		
		r=rand()%3;
		if(r==0) x2=x1+1, y2=y1+1, x1--, y1--;
		else if(r==1) x2=x1+2, y2=y1, x1-=2;
		else          y2=y1+2, x2=x1, y1-=2;

		if(x1<1) x1=1;
		if(y1<1) y1=1;
		if(x2>MAP_SIZE-2) x2=MAP_SIZE-2;
		if(y2>MAP_SIZE-2) y2=MAP_SIZE-2;
		Room(x1,y1,x2,y2,0);

/*		x1=rand()%(MAP_SIZE-2)+1;
		y1=rand()%(MAP_SIZE-2)+1;
		x2=rand()%(MAP_SIZE-2)+1;
		y2=rand()%(MAP_SIZE-2)+1;
		if(x2-x1>3 || y2-y1>3)
			continue;
		Room(x1,y1,x2,y2,0);*/
	}
	
	ClearMark();
	FloodMark(start_x,start_y,T_FLOOR);
	for(x=0;x<MAP_SIZE;x++)
		for(y=0;y<MAP_SIZE;y++)
			if(!tiles[x][y].mark)
				tiles[x][y].type=T_WALL;

	for(x=0;x<MAP_SIZE;x++)
		for(y=0;y<MAP_SIZE;y++)
			CreateTile(x,y);
				
	spawn_player();
}

tObject *tile_object(int x,int y,char *name,int side)
{
	tObject *o = create_object();
	o->mesh = load_mesh(name,0.01);
	o->heading = side*90;
	o->pos = vec3(x+((side==1||side==2)?0.999:0),y+((side==2||side==3)?0.999:0),0);
	o->flags = FL_STATIC;
	update_object(o);

	return o;
}

void CMap::CreateTile(int x,int y)
{
    tObject *o;
    int i;

	if(tiles[x][y].type==T_FLOOR)
	{
		o = tile_object(x,y,"floor1",0);
		o->tex = load_texture("brick1");
		o->flags |= FL_NO_COLLISION;
		
		if(rand()%100<(10+3*level) && (x!=start_x || y!=start_y))
			spawn_enemy(x,y);

		if(rand()%100<5 && (x!=start_x || y!=start_y))
			spawn_cloud(x,y);
			
		for(i=0;i<4;i++)
			if(tiles[x+DX[i]][y+DY[i]].type==T_WALL)
			{
				o = tile_object(x,y,"wall1",i);
				o->tex = load_texture("brick1");
				o->flags |= FL_NO_COLLISION;

				if(rand()%100<20)
				{
					o = tile_object(x,y,"torch1",i);
					o->tex = load_texture("brick1");
					o->flags |= FL_NO_COLLISION;

					o = create_object();
					o->flags |= FL_LIGHT | FL_EMIT | FL_STATIC | FL_NO_COLLISION;
					o->ptype = part_type("fire");

					o->ppr = 0.005;
					o->pvr = 0.00;
					o->pvel = vec3(0,0,0.15);
					o->pscale = 1;

					o->think_func = think_torch;

					o->pos = vec3(x+0.5+0.47*DX[i],y+0.5+0.47*DY[i],0.29);
					update_object(o);
				}
			}
	}
	
	if(tiles[x][y].type==T_WALL)
	{
		o = tile_object(x,y,"floor1",0);
		o->tex = load_texture("brick1");
		o->flags |= FL_NO_COLLISION;
		o->pos.z = 0.5;
	}
}


void make_map(int seed)
{
	int ns = rand();
	map.Make(seed);
	srand(ns);
}


void render_tile(int x,int y)
{
	if(x<0 || y<0 || x>=MAP_SIZE || y>=MAP_SIZE)
		return;
	
/*	if(!tex) tex = load_texture("brick1");
	
	glPushMatrix();
	glTranslatef(x,y,0);

	     if(map.tiles[x][y].type==0) glColor3f(0.1,0.1,0.1);
	else if(map.tiles[x][y].type==1) glColor3f(1.0,1.0,1.0);
	else if(map.tiles[x][y].type==2) glColor3f(0.0,1.0,0.0);
	else if(map.tiles[x][y].type==3) glColor3f(0.3,0.3,0.3);
	else if(map.tiles[x][y].type==4) glColor3f(0.3,0.3,0.3);
	else 							 glColor3f(1.0,0.0,1.0);

	tex->Bind(0);
	glBegin(GL_QUADS);
#define v(x,y,z)	glTexCoord2f(x,y); glVertex3f(x,y,z);
	v(0,0,0) v(0,1,0) v(1,1,0) v(1,0,0)
#undef v
	glEnd();
	tex->UnBind(0);

	glPopMatrix();*/

//	glColor3f(1,1,1);

	map.tiles[x][y].render_frame=render_frame;
	
	render_objects(x,y);
}

