

#include "allstuff.h"


vector<CMesh *> meshes;
CMesh dummy_mesh;

CTexture *texture_repl = NULL;



int model_arbvp=0;



struct tVertex {
	short x,y,z,n;
};


bool CFrame::Load(char *path,int id,int n_verts,float scale)
{
	char fpath[100];
	FILE *fp;
	int i;
	
	if(n_verts<=0) return false;

	sprintf(fpath,"%sframe%02d.dat",path,id);
	fp = fopen(fpath,"rb");
	if(!fp) return false;

	tVertex *vt = new tVertex[n_verts];
	assert(vt);

	fread(vt,1,sizeof(tVertex)*n_verts,fp);
	fclose(fp);

	xyz = new vec3[n_verts];
	assert(xyz);
	
	normals = new vec3[n_verts];
	assert(normals);
	
/*	fread(xyz,1,sizeof(vec3)*n_verts,fp);
	fclose(fp);
	
	sprintf(fpath,"%sframe%02dn.dat",path,id);
	fp = fopen(fpath,"rb");
	if(!fp) return false;

	fread(normals,1,sizeof(vec3)*n_verts,fp);
	fclose(fp);*/
	
	float lng,lat;
	for(i=0;i<n_verts;i++)
	{
//		xyz[i].x*=scale;
//		xyz[i].y*=scale;
//		xyz[i].z*=scale;
		xyz[i].x=vt[i].x/64.0*scale;
		xyz[i].y=vt[i].y/64.0*scale;
		xyz[i].z=vt[i].z/64.0*scale;
		
		lng = (vt[i].n&0xFF)*2*M_PI/255.0;
		lat = ((vt[i].n>>8)&0xFF)*2*M_PI/255.0;
		normals[i].x = cos(lat)*sin(lng);
		normals[i].y = sin(lat)*sin(lng);
		normals[i].z = cos(lng);
	}
	delete vt;
	
	glGenBuffersARB(1,(GLuint*)&vb_xyz);
	glBindBufferARB(GL_ARRAY_BUFFER,vb_xyz);
	glBufferDataARB(GL_ARRAY_BUFFER,sizeof(vec3)*n_verts,xyz,GL_STATIC_DRAW);

	glGenBuffersARB(1,(GLuint*)&vb_normals);
	glBindBufferARB(GL_ARRAY_BUFFER,vb_normals);
	glBufferDataARB(GL_ARRAY_BUFFER,sizeof(vec3)*n_verts,normals,GL_STATIC_DRAW);

	return true;
}


bool CMesh::Load(char *_name,float scale)
{
	char path[100],fpath[100];
	FILE *fp;
	int i;

	// make path
	strcpy(name,_name);
	sprintf(path,"data/mdl/%s/",name);

	// load surfaces
	sprintf(fpath,"%smesh.dat",path);
	fp = fopen(fpath,"rb");
	if(!fp) return false;

	fseek(fp,0,SEEK_END);
	n_surf = ftell(fp)/sizeof(tMesh);
	if(n_surf<=0) { fclose(fp); return false; }
	
	surf = new tMesh[n_surf];
	assert(surf);

	fseek(fp,0,SEEK_SET);
	fread(surf,1,sizeof(tMesh)*n_surf,fp);
	fclose(fp);

	// count vertexes and faces
	n_verts=n_faces=0;
	for(i=0;i<n_surf;i++)
	{
		n_verts += surf[i].n_verts;
		n_faces += surf[i].n_faces;
	}
	
	if(n_verts<=0 || n_faces<=0)
		return false;

	// load indexes
	indexes = new int[n_faces*3];
	assert(indexes);
	
	sprintf(fpath,"%striangles.dat",path);
	fp = fopen(fpath,"rb");
	if(!fp) return false;
	fread(indexes,1,sizeof(int)*3*n_faces,fp);
	fclose(fp);
	
	// load texcoords
	st = new vec2[n_verts];
	assert(st);
	
	sprintf(fpath,"%stexcoord.dat",path);
	fp = fopen(fpath,"rb");
	if(!fp) return false;
	fread(st,1,sizeof(vec2)*n_verts,fp);
	fclose(fp);
	
	// load frames
	CFrame *f;
	int id=0;
	
	while(1)
	{
		f = new CFrame();
		if(!f->Load(path,id,n_verts,scale))
		{
			delete f;
			break;
		}
		frames.push_back(f);
		id++;
	}
	if(id==0) return false;
	
	// load textures
	tex = new CTexture*[n_surf];
	for(i=0;i<n_surf;i++)
	{
		sprintf(path,"%s_%s",name,surf[i].name);
		tex[i] = load_texture(path);
	}
	
	// calc bounding box
	vec3 v;
	box_min=vec3( 1e9, 1e9, 1e9);
	box_max=vec3(-1e9,-1e9,-1e9);
	for(i=0;i<n_verts;i++)
	{
		v=frames[0]->xyz[i];
		if(v.x<box_min.x) box_min.x=v.x;
		if(v.y<box_min.y) box_min.y=v.y;
		if(v.z<box_min.z) box_min.z=v.z;
		if(v.x>box_max.x) box_max.x=v.x;
		if(v.y>box_max.y) box_max.y=v.y;
		if(v.z>box_max.z) box_max.z=v.z;
	}

	if(frames.size()!=1)
	{
		float r=MAX(MAX(-box_min.x,-box_min.y),MAX(box_max.x,box_max.y));
		box_min.x=-r;
		box_min.y=-r;
		box_max.x= r;
		box_max.y= r;
	}
	
	
	glGenBuffersARB(1,(GLuint*)&vb_indexes);
	glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER,vb_indexes);
	glBufferDataARB(GL_ELEMENT_ARRAY_BUFFER,sizeof(int)*3*n_faces,indexes,GL_STATIC_DRAW);

	glGenBuffersARB(1,(GLuint*)&vb_st);
	glBindBufferARB(GL_ARRAY_BUFFER,vb_st);
	glBufferDataARB(GL_ARRAY_BUFFER,sizeof(vec2)*n_verts,st,GL_STATIC_DRAW);

	return true;
}


CMesh *load_mesh(char *name,float scale)
{
	int i;
	for(i=0;i<meshes.size();i++)
		if(strcmp(meshes[i]->name,name)==0)
			return meshes[i];

	CMesh *m = new CMesh();
	assert(m);
	
	if(!m) return &dummy_mesh;
	if(!m->Load(name,scale))
	{
		allegro_message("Can't load mesh: %s",name);
		delete m;
		return &dummy_mesh;
	}
	
	//TODO: remove!
	char path[100];
	for(i=0;i<m->n_surf;i++)
	{
		sprintf(path,"work/%s_%s.bmp",name,m->surf[i].name);
		make_surf_texture(m,i,path);
	}
	
	meshes.push_back(m);
	return m;
}

void unload_meshes()
{
	int i;
	for(i=0;i<meshes.size();i++)
		if(meshes[i])
			delete meshes[i];
	meshes.clear();
}

void mesh_surface(CMesh *m,int s,int fr)
{
	if(!m->surf) return;

	if(s>=m->n_surf) return;
	if(fr>=m->frames.size()) return;
	
	if(texture_repl)
		texture_repl->Bind(0);
	else
		if(m->tex && m->tex[s])
			m->tex[s]->Bind(0);

	int f,fe,v;
	vec3 *vtx = m->frames[fr]->xyz;
	vec3 *nrm = m->frames[fr]->normals;
	vec2 *st = m->st;

	f = 3*m->surf[s].o_faces;
	fe = f + 3*m->surf[s].n_faces;

//	glColor3f(1,1,1);

	glBindBufferARB(GL_ARRAY_BUFFER,m->frames[fr]->vb_xyz);
	glVertexPointer(3,GL_FLOAT,0,NULL);
	glEnableClientState(GL_VERTEX_ARRAY);
	
	glBindBufferARB(GL_ARRAY_BUFFER,m->frames[fr]->vb_normals);
	glNormalPointer(GL_FLOAT,0,NULL);
	glEnableClientState(GL_NORMAL_ARRAY);

	glBindBufferARB(GL_ARRAY_BUFFER,m->vb_st);
	glTexCoordPointer(2,GL_FLOAT,0,NULL);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER,m->vb_indexes);
	
//	glEnable(GL_VERTEX_PROGRAM_ARB);
	
//	glEnable(GL_LIGHTING);
	glDrawElements(GL_TRIANGLES,3*m->surf[s].n_faces,GL_UNSIGNED_INT,
		((int*)NULL)+3*m->surf[s].o_faces);
//	glDisable(GL_LIGHTING);

//	glDisable(GL_VERTEX_PROGRAM_ARB);



/*
//	glBegin(GL_TRIANGLES);
	glBegin(GL_LINES);
	while(f<fe)
	{
		v=m->indexes[f];
//		glTexCoord2f(st[v].x,st[v].y);
//		glColor3f(nrm[v].x*10.5+0.5,nrm[v].y*10.5+0.5,nrm[v].z*10.5+0.5);
//		glColor3f(nrm[v].z>0,0,0);
		glColor3f(1,0,0);
		glVertex3f(vtx[v].x+nrm[v].x*0.03,vtx[v].y+nrm[v].y*0.03,vtx[v].z+nrm[v].z*0.03);
		glColor3f(0,0,1);
		glVertex3f(vtx[v].x,vtx[v].y,vtx[v].z);
		f++;
	}
	glEnd();
*/

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	if(texture_repl)
		texture_repl->UnBind(0);
	else
		if(m->tex && m->tex[s])
			m->tex[s]->UnBind(0);
}

void render_mesh(CMesh *m,int fr)
{
	int s;
	for(s=0;s<m->n_surf;s++)
		mesh_surface(m,s,fr);
}


void tex_line(BITMAP *bmp,vec2 *st,int v1,int v2)
{
	int x1,y1,x2,y2,dx,dy;
	x1=int(st[v1].x*256);
	y1=int(st[v1].y*256);
	x2=int(st[v2].x*256);
	y2=int(st[v2].y*256);
	for(dy=-1;dy<=1;dy++)
		for(dx=-1;dx<=1;dx++)
			line(bmp,x1+256*dx,y1+256*dy,x2+256*dx,y2+256*dy,makecol32(255,255,255));
}

void make_surf_texture(CMesh *m,int s,char *path)
{
	if(!m->surf) return;

	if(s>=m->n_surf) return;
	if(s) return;

	BITMAP *tex = create_bitmap(256,256);
	if(!tex) return;
	
	int f,fe,v0,v1,v2;
	vec2 *st = m->st;

	f = 3*m->surf[s].o_faces;
	fe = f + 3*m->surf[s].n_faces;

	while(f<fe)
	{
		v0=m->indexes[f+0];
		v1=m->indexes[f+1];
		v2=m->indexes[f+2];
		tex_line(tex,st,v0,v1);
		tex_line(tex,st,v1,v2);
		tex_line(tex,st,v2,v0);
		f+=3;
	}

	save_bitmap(path,tex,NULL);
	destroy_bitmap(tex);
}

void set_light(int id,vec3 pos,vec3 color)
{
/*	float _mv[16];
	MATRIX_f mv;
	vec3 p;

	glGetFloatv(GL_MODELVIEW_MATRIX,_mv);
	allegro_gl_GLfloat_to_MATRIX_f((GLfloat*)_mv,&mv);
	apply_matrix_f(&mv,pos.x,pos.y,pos.z,&p.x,&p.y,&p.z);


	glProgramLocalParameter4fARB(GL_VERTEX_PROGRAM_ARB,id  ,p.x,p.y,p.z,1);
	glProgramLocalParameter4fARB(GL_VERTEX_PROGRAM_ARB,id+8,color.x,color.y,color.z,0);*/
	
	float p[4]={pos.x,pos.y,pos.z,1};
	float c[4]={color.x,color.y,color.z,1};
	glLightfv(GL_LIGHT0+id,GL_POSITION,p);
	glLightfv(GL_LIGHT0+id,GL_DIFFUSE,c);
	glEnable(GL_LIGHT0+id);
}

void unset_light(int id)
{
	glDisable(GL_LIGHT0+id);
}

void load_arbvp()
{
/*	glGenProgramsARB(1,(GLuint*)&model_arbvp);
	glBindProgramARB(GL_VERTEX_PROGRAM_ARB,model_arbvp);
	
	char buff[65000];
	FILE *fp = fopen("data/model.avp","rt");
	if(!fp) return;
	int len = fread(buff,1,65000,fp);
	fclose(fp);
	
	glProgramStringARB(GL_VERTEX_PROGRAM_ARB,GL_PROGRAM_FORMAT_ASCII_ARB,len,buff);

	set_light(0,vec3(1,0,1),vec3(1,1,1));*/
}

void init_lighting()
{
	int i;
	float one[4]={1,1,1,1};
	float zero[4]={0,0,0,0};
	float amb[4]={0.15,0.15,0.15,1.0};

	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,one);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,zero);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,15);

	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,amb);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);

	for(i=0;i<8;i++)
	{
//		set_light(i,vec3(0,0,0),vec3(0,0,0));
		glLightfv(GL_LIGHT0+i,GL_AMBIENT,zero);
		glLightfv(GL_LIGHT0+i,GL_DIFFUSE,zero);
		glLightfv(GL_LIGHT0+i,GL_SPECULAR,zero);
		glLightf(GL_LIGHT0+i,GL_CONSTANT_ATTENUATION,1.0);
		glLightf(GL_LIGHT0+i,GL_LINEAR_ATTENUATION,0.0);
		glLightf(GL_LIGHT0+i,GL_QUADRATIC_ATTENUATION,1.0);
	}
}

