

#ifndef _MESH_H
#define _MESH_H


#define MAX_LIGHTS		8




struct vec2 {
	float x,y;
};


struct tMesh {
	char	name[64];
	int		n_faces,o_faces;
	int		n_verts,o_verts;
};


struct CFrame {
	vec3	*xyz;
	vec3	*normals;
	
	int		vb_xyz, vb_normals;
	
	CFrame() : xyz(0), normals(0), vb_xyz(0), vb_normals(0) {}
	~CFrame()
	{
		if(xyz) delete xyz;
		if(normals) delete normals;
		if(vb_xyz) glDeleteBuffersARB(1,(GLuint*)&vb_xyz);
		if(vb_normals) glDeleteBuffersARB(1,(GLuint*)&vb_normals);
	}

	bool Load(char *path,int id,int n_verts,float scale);
};

struct CMesh {
	char name[64];
	vector<CFrame *> frames;
	
	int n_surf;
	int n_verts;
	int n_faces;
	
	vec3	box_min, box_max;


	tMesh		*surf;
	int			*indexes;
	vec2		*st;
	
	int			vb_indexes, vb_st;
	
	CTexture	**tex;


	CMesh() : surf(0), indexes(0), st(0), tex(0), vb_indexes(0), vb_st(0) {}
	~CMesh()
	{
		int i;
		for(i=0;i<frames.size();i++)
			if(frames[i])
				delete frames[i];
		if(surf) delete surf;
		if(indexes) delete indexes;
		if(st) delete st;
		if(tex) delete tex;
		if(vb_indexes) glDeleteBuffersARB(1,(GLuint*)&vb_indexes);
		if(vb_st) glDeleteBuffersARB(1,(GLuint*)&vb_st);
	}
	
	bool Load(char *name,float scale);
};


extern CTexture *texture_repl;


CMesh *load_mesh(char *name,float scale);
void unload_meshes();
void mesh_surface(CMesh *m,int s,int f);
void render_mesh(CMesh *m,int fr);
void make_surf_texture(CMesh *m,int s,char *path);
void set_light(int id,vec3 pos,vec3 color);
void unset_light(int id);
void load_arbvp();
void init_lighting();



#endif

