

#include "allstuff.h"


tObject *player_obj;
vec3 mouse_p0;
vec3 mouse_p1;
vec3 cam_follow;
tObject *col_object;



void clear_objects()
{
	int x,y;
	tObject *o,*n;
	
	for(y=0;y<MAP_SIZE;y++)
		for(x=0;x<MAP_SIZE;x++)
		{
			o=map.tiles[x][y].first;
			while(o)
			{
				n=o->next;
				delete o;
				o=n;
			}
			map.tiles[x][y].first=NULL;
			map.tiles[x][y].last=NULL;
		}
}

void bind_object(tObject *o)
{
	o->tile_x = MID(0,int(floor(o->pos.x)),MAP_SIZE-1);
	o->tile_y = MID(0,int(floor(o->pos.y)),MAP_SIZE-1);

	tTile *t = &(map.tiles[o->tile_x][o->tile_y]);
	if(t->last) t->last->next = o;
	else		t->first = o;
	o->prev = t->last;
	o->next = NULL;
	t->last = o;
}

void unbind_object(tObject *o)
{
	tTile *t = &(map.tiles[o->tile_x][o->tile_y]);
	if(o->next) o->next->prev = o->prev;
	else		t->last = o->prev;
	if(o->prev) o->prev->next = o->next;
	else		t->first = o->next;
}

void update_object(tObject *o)
{
	if(o->tile_x!=int(floor(o->pos.x)) || o->tile_y!=int(floor(o->pos.y)))
	{
		unbind_object(o);
		bind_object(o);
	}
	if(o->mesh)
	{
		o->box_min = o->mesh->box_min;
		o->box_max = o->mesh->box_max;
	}
}

tObject *create_object()
{
	tObject *o = new tObject();
	memset(o,0,sizeof(*o));
	o->r=o->g=o->b=o->a=1;
	bind_object(o);
	return o;
}

void destroy_object(tObject *o)
{
	unbind_object(o);
	delete o;
}

void render_object(tObject *o)
{
	if(o->mesh)
	{
		glPushMatrix();
		glTranslatef(o->pos.x,o->pos.y,o->pos.z);
		glRotatef(o->heading,0,0,1);
		texture_repl = o->tex;
		if(!((o->flags&FL_FULL_BRIGHT)||(o->flags&FL_LIGHT)))
			glEnable(GL_LIGHTING);
			
/*		if(o->flags&FL_BLEND_ALPHA)
		{
//			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
			glDepthMask(false);
		}*/
			
		glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,&(o->r));
		render_mesh(o->mesh,o->frame);

/*		if(o->flags&FL_BLEND_ALPHA)
		{
			glDisable(GL_BLEND);
			glDepthMask(true);
		}*/
		
/*		glBegin(GL_LINES);

#define _bx(v)	(v?(o->box_max.x):(o->box_min.x))
#define _by(v)	(v?(o->box_max.y):(o->box_min.y))

#define v(x,y)	glVertex3f(_bx(x),_by(y),o->box_min.z); glVertex3f(_bx(x),_by(y),o->box_max.z);
		v(0,0) v(0,1) v(1,1) v(1,0)
#undef v

#define v(x,y)	glVertex3f(_bx(x),_by(y),o->box_min.z);
		v(0,0) v(0,1) v(0,1) v(1,1) v(1,1) v(1,0) v(1,0) v(0,0)
#undef v

#define v(x,y)	glVertex3f(_bx(x),_by(y),o->box_max.z);
		v(0,0) v(0,1) v(0,1) v(1,1) v(1,1) v(1,0) v(1,0) v(0,0)
#undef v

		glEnd();*/

		if(!((o->flags&FL_FULL_BRIGHT)||(o->flags&FL_LIGHT)))
			glDisable(GL_LIGHTING);
		texture_repl = NULL;
		glPopMatrix();
	}
}

int gather_lights(int id,int tx,int ty)
{
	if(tx<0 || ty<0 || tx>=MAP_SIZE || ty>=MAP_SIZE)
		return id;
		
	tObject *o;
	o=map.tiles[tx][ty].first;
	while(o && id<MAX_LIGHTS)
	{
		if(o->flags & FL_LIGHT)
			set_light(id++,o->pos,o->light_color);
		o=o->next;
	}

	return id;
}

void render_objects(int tx,int ty)
{
	if(map.tiles[tx][ty].render_frame!=render_frame)
		return;

	tObject *o;
	int x,y,id;

	id=0;
	for(y=ty-2;y<=ty+2;y++)
		for(x=tx-2;x<=tx+2;x++)
			id=gather_lights(id,x,y);

	while(id<MAX_LIGHTS)
		unset_light(id++);

	o=map.tiles[tx][ty].first;
	while(o)
	{
		render_object(o);
		o=o->next;
	}
	
	float one[4]={1,1,1,1};
	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,one);
}

bool check_collision(tObject *col,int tx,int ty)
{
	vec3 a1,a2,b1,b2;
	int ok=0;
	b1=col->pos+col->box_min;
	b2=col->pos+col->box_max;
	
	if(int(floor(b1.x))==tx) ok|=1;
	if(int(floor(b1.y))==ty) ok|=2;
	if(int(floor(b2.x))==tx) ok|=1;
	if(int(floor(b2.y))==ty) ok|=2;

	if(ok==3)
	{
		if(tx<0 || ty<0 || tx>=MAP_SIZE || ty>=MAP_SIZE)
			return true;
		if(map.tiles[tx][ty].type==T_WALL)
			return true;
		if(col->flags&FL_WALK && map.tiles[tx][ty].type!=T_FLOOR)
			return true;
	}

	tObject *o;

	o=map.tiles[tx][ty].first;
	for(;o;o=o->next)
	{
		if(o==col)
			continue;
		if(o->flags&FL_NO_COLLISION)
			continue;
		if(o->owner == col || col->owner == o)
			continue;
			
		a1=o->pos+o->box_min;
		a2=o->pos+o->box_max;
		if(a2.x>=b1.x && b2.x>=a1.x)
			if(a2.y>=b1.y && b2.y>=a1.y)
				if(a2.z>=b1.z && b2.z>=a1.z)
				{
					col_object = o;
					return true;
				}
	}

	return false;
}

bool check_collision(tObject *col)
{
	col_object = NULL;
	int tx=int(floor(col->pos.x-0.5));
	int ty=int(floor(col->pos.y-0.5));
	if(check_collision(col,tx  ,ty  )) return true;
	if(check_collision(col,tx+1,ty  )) return true;
	if(check_collision(col,tx  ,ty+1)) return true;
	if(check_collision(col,tx+1,ty+1)) return true;
	return false;
}

void think_object(tObject *o,float dt)
{
	if(o->flags&FL_EMIT && o->ptype)
	{
		o->emit_timer+=((o->flags&FL_FAST_EMIT)?2*dt:dt);
		if(o->emit_timer>0.1)
		{
			add_parts(o->ptype,o->pos,o->pvel,o->ppr,o->pvr,1,o->pscale);
			o->emit_timer-=0.1;
		}
	}
	
	if(o->think_func)
	{
		o->think_time-=dt;
		if(o->think_time<=0)
		{
			o->think_time=1e9;
			o->think_func(o);
		}
	}

	if(o->flags&FL_STATIC)
		return;

	if(o->flags&FL_GRAVITY)
		o->vel.z-=1.0*dt;
	o->pos.z+=o->vel.z;
	if(o->pos.z<0) o->pos.z=0;
	
	vec3 oldpos=o->pos;
	o->pos.x+=o->vel.x*dt;
	int hit=0;
	if(check_collision(o))
	{
		if(!(o->flags&FL_TRIGGER))
			o->pos = oldpos;
		if(o->col_func)
			o->col_func(o);
		hit=1;
	}
		
	oldpos=o->pos;
	o->pos.y+=o->vel.y*dt;
	if(check_collision(o) && !(o->flags&FL_TRIGGER))
	{
		if(!(o->flags&FL_TRIGGER))
			o->pos = oldpos;
		if(o->col_func && !hit)
			o->col_func(o);
	}

	if(o->flags & FL_DELETED)
		destroy_object(o);
}

void think_objects(float dt)
{
	tObject *o,*n;
	int tx,ty;
	static int tf=0;

	tf++;
	for(ty=0;ty<MAP_SIZE;ty++)
		for(tx=0;tx<MAP_SIZE;tx++)
		{
			if(map.tiles[tx][ty].render_frame!=render_frame)
				continue;

			o=map.tiles[tx][ty].first;
			while(o)
			{
				n=o->next;
				if(o->think_frame!=tf)
				{
					o->think_frame=tf;
					think_object(o,dt);
				}
				o=n;
			}
		}

	for(ty=0;ty<MAP_SIZE;ty++)
		for(tx=0;tx<MAP_SIZE;tx++)
		{
			if(map.tiles[tx][ty].render_frame!=render_frame)
				continue;

			o=map.tiles[tx][ty].first;
			while(o)
			{
				update_object(o);
				o=o->next;
			}
		}
}

vec3 mouse_plane(float p)
{
	float xp=-float(mouse_x)/SCREEN_W*2+1;
	float yp= float(mouse_y)/SCREEN_H*2-1;
	xp/=1.2;
	yp/=1.2;
//	float ax = cam_ax*M_PI/180.0 + atan(xp/tan(70));
//	float ay = cam_ay*M_PI/180.0 + atan(yp/tan(70));
	float ax = cam_ax*M_PI/180.0;
	float ay = cam_ay*M_PI/180.0 + atan(yp/tan(70));

	vec3 v;
	v.x =  (cos(ax)*cos(ay)-xp*sin(ax))/sin(ay);
	v.y =  (sin(ax)*cos(ay)+xp*cos(ax))/sin(ay);

	vec3 m_pos;
	m_pos.x = cam_pos.x+v.x*(cam_pos.z-p);
	m_pos.y = cam_pos.y+v.y*(cam_pos.z-p);
	m_pos.z = 0;
	
/*	static CMesh *m=NULL;
	if(!m) m=load_mesh("druid",0.005);
	glPushMatrix();
	glTranslatef(m_pos.x,m_pos.y,m_pos.z);
	glColor4f(1,1,1,1);
	render_mesh(m,0);
	glPopMatrix();*/
	
	return m_pos;
}

void cam_follow_player()
{
	float dx=cos(cam_ax/180.0*M_PI),dy=sin(cam_ax/180.0*M_PI);
	cam_follow = cam_follow*0.8 + player_obj->pos*0.2;
	cam_ay=30;
	cam_pos=cam_follow + vec3(-dx,-dy,1)*1;
	
	mouse_p0 = mouse_plane(0.00);
	mouse_p1 = mouse_plane(0.15);
}

