

#include "allstuff.h"


#define MAX_PARTS	1024
#define MAX_PTYPES	256


struct tParticle;

struct tPType {
	char *name;
	int flags;
	char *tname;
	float r,g,b,a,s;
	float time;
	CTexture *tex;
};

struct tParticle {
	tPType *pt;
	vec3 pos,vel;
	float time;
	float scale;
};


#define PFL_GRAVITY		(1<<0)
#define PFL_END			(1<<1)
#define PFL_ALPHA		(1<<2)



tParticle parts[MAX_PARTS];
tPType ptypes[MAX_PTYPES]=
{
{ 0 },
// name        flags      tname       r   g   b   a   s      time
{ "fire"	,0			,"part1"	,1.0,0.8,0	,1.0,0.008	, 1.0 },
{ 0			,PFL_END	,0			,0	,0	,0	,1.0,0.0	, 0.0 },
{ "sfire"	,0			,"part1"	,1.0,0.8,0	,1.0,0.008	, 0.3 },
{ 0			,PFL_ALPHA	,"part1"	,0.0,0.0,0	,1.0,0.002	, 1.0 },
{ 0			,PFL_END	,0			,0	,0	,0	,1.0,0.0	, 0.0 },
{ "scold"	,0			,"part1"	,0.0,1.0,1.0,1.0,0.008	, 0.3 },
{ 0			,PFL_END	,0			,0	,0	,0	,1.0,0.0	, 0.0 },
{ "sacid"	,0			,"part1"	,0.0,1.0,0.0,1.0,0.008	, 0.3 },
{ 0			,PFL_END	,0			,0	,0	,0	,1.0,0.0	, 0.0 },
{ "exit"	,0			,"part1"	,0.0,1.0,1.0,1.0,0.1	, 1.0 },
{ 0			,PFL_END	,0			,0	,0	,0	,1.0,0.0	, 0.0 },
{ "cloud"	,0			,"part1"	,0.0,1.0,0.0,1.0,0.1	, 1.0 },
{ 0			,PFL_END	,0			,0	,0	,0	,1.0,0.0	, 0.0 },
};
int part_alloc=0;
int n_parts=0;


void render_parts()
{
	int i;
	tPType *pt=NULL;
	tParticle *pr;
	float p;
	float r1,b1,g1,a1,s1;
	float r2,b2,g2,a2,s2;

	glPointParameterfARB(GL_POINT_SIZE_MIN_ARB,0);
	glPointParameterfARB(GL_POINT_SIZE_MAX_ARB,1000);
	glPointParameterfARB(GL_POINT_FADE_THRESHOLD_SIZE_ARB,1);
	
	float da[]={0,0,1.0/(SCREEN_W*SCREEN_W)};
	glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB,da);
	
	glTexEnvi(GL_POINT_SPRITE_ARB,GL_COORD_REPLACE_ARB,GL_TRUE);
	glEnable(GL_POINT_SPRITE_ARB);

	glEnable(GL_BLEND);
	glDepthMask(false);

	pr=parts;
	for(i=1;i<MAX_PARTS;i++,pr++)
	{
		if(!pr->pt)
			continue;

		if(pt!=pr->pt)
		{
			pt=pr->pt;
			r1=pt->r; r2=(pt+1)->r-pt->r;
			g1=pt->g; g2=(pt+1)->g-pt->g;
			b1=pt->b; b2=(pt+1)->b-pt->b;
			a1=pt->a; a2=(pt+1)->a-pt->a;
			s1=pt->s; s2=(pt+1)->s-pt->s;
			if(pt->tex)
				pt->tex->Bind(0);
				
			if(pt->flags&PFL_ALPHA)
				glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
			else
				glBlendFunc(GL_SRC_ALPHA,GL_ONE);
		}


		p=pr->time/pt->time;
		da[2]=1.0/((s1+p*s2)*pr->scale*SCREEN_W*SCREEN_W);
		glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB,da);
		glBegin(GL_POINTS);
		glColor4f(r1+p*r2,g1+p*g2,b1+p*b2,a1+p*a2);
		glVertex3f(pr->pos.x,pr->pos.y,pr->pos.z);
		glEnd();
	}

	if(pt && pt->tex)
		pt->tex->UnBind(0);

	glDepthMask(true);
	glDisable(GL_BLEND);

	glTexEnvi(GL_POINT_SPRITE_ARB,GL_COORD_REPLACE_ARB,GL_FALSE);
	glDisable(GL_POINT_SPRITE_ARB);
}

void think_parts(float dt)
{
	int i;
	tPType *pt;
	tParticle *pr = parts;
	
	for(i=0;i<MAX_PARTS;i++,pr++)
	{
		if(!pr->pt)
			continue;
		pt=pr->pt;
		
		if(pt->flags&PFL_GRAVITY)
			pr->vel.z -= dt;
		pr->pos += pr->vel*dt;
		pr->time+=dt;

		if(pr->time<pt->time)
			continue;

//		if(pr->next) pr->next->prev=pr->prev;
//		else 	pt->last = pr->prev;
//		if(pr->prev) pr->prev->next=pr->next;
//		else	pt->first = pr->next;

		pr->time-=pt->time;
		pr->pt++;
		if(pr->pt->flags&PFL_END)
		{
			pr->pt = NULL;
			n_parts--;
			continue;
		}
//		pt=pr->pt;
//		pr->next = NULL;
//		pr->prev = pt->last;
//		if(pr->prev) pr->prev->next = pr;
//		else pt->first = pr;
//		pt->last = pr;

	}
}

void clear_parts()
{
	int i;
	
	for(i=0;i<MAX_PTYPES;i++)
	{
//		ptypes[i].first = NULL;
//		ptypes[i].last = NULL;
	}

	for(i=0;i<MAX_PARTS;i++)
	{
		parts[i].pt = NULL;
//		parts[i].next = NULL;
//		parts[i].prev = NULL;
	}
	
	part_alloc = 0;
	n_parts=0;
}

void init_parts()
{
	int i;

	for(i=0;i<MAX_PTYPES;i++)
		if(ptypes[i].tname)
			ptypes[i].tex = load_texture(ptypes[i].tname);
		
	clear_parts();
}

void add_parts(tPType *pt,vec3 pos,vec3 vel,float rp,float rv,int cnt,float scale)
{
	tParticle *pr;

	if(!pt) return;
	
	while(cnt-->0 && n_parts<MAX_PARTS)
	{
		while(parts[part_alloc].pt)
			part_alloc=(part_alloc+1)%MAX_PARTS;
			
		pr=parts+part_alloc;
		n_parts++;
		
		pr->pt = pt;
//		pr->next = NULL;
//		pr->prev = ptypes[type].last;
//		if(pr->prev) pr->prev->next = pr;
//		else ptypes[type].first = pr;
//		ptypes[type].last = pr;
		pr->time=0;
		pr->scale=scale;

#define rnd		vec3((rand()%201-100)*0.01,(rand()%201-100)*0.01,(rand()%201-100)*0.01)
		pr->pos = pos + rnd*rp;
		pr->vel = vel + rnd*rv;
#undef rnd
	}
}

tPType *part_type(char *name)
{
	int i;
	for(i=0;i<MAX_PTYPES;i++)
		if(ptypes[i].name && strcmp(ptypes[i].name,name)==0)
			return ptypes+i;
	return NULL;
}

